#!/bin/bash
#
# Fixes the lexers generated by bison/flex so they don't have exit()
# statements (which we won't hit anyway)

set -e

for fname in src/vendor/io/*-lexer.c; do
    # Remove unused exit points from the lexer
    cat ${fname} | grep -v '^\s*exit.*YY_EXIT_FAILURE' >${fname}.new
    mv ${fname}.new ${fname}

    # Remove 'noreturn' declaration from exit points
    sed -e 's/^\([^#]*\)yynoreturn /\1__attribute__((unused)) /g' ${fname} >${fname}.new
    mv ${fname}.new ${fname}
done

for fname in src/vendor/io/*-parser.c; do
    # Bison 3.8.2 (and maybe other versions) refer to #include yy.tab.h in the
    # generated file; we need to replace this with the real header name
    header=`basename ${fname} .c`.h
    cat ${fname} | sed -e 's,^#include.*yy.tab.h.*$,#include "io/parsers/'"${header}"'",g' >${fname}.new
    mv ${fname}.new ${fname}
done

